/* 	CalendarPane.java
 * 
 * 	Created on: Jun 1, 2003
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2003-2007, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
* [When]		[Who]				[What]
* 29-Sep-03	    dennis@brains2b.nl	Implemented keyboard handling
* 04-10-2003	dennis@brains2b.nl	Implemented all keyboard handling
* 06-07-2004	dennis@brains2b.nl	Fixed problem with weeknumbers
* 30-06-2007    dennis@brains2b.nl  Changed setMiddle(..) method (TwelBox 0.40)
*/
package org.brains2b.swingext.dialog;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.SystemColor;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.DateFormatSymbols;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;

/** Pane for showing a Calendar and selecting a date.
 *  <p>Shows a grid in which to select a Date and change months and years
 *  <p>If no date is supplied at startup the current date will be used as starting point
 *  <p>Supports full internationalisation.
 *  <LU>You can start this panel conviniently through
 *  <LI><code>showDialog(Component parent,java.util.Date date,String title)</code>
 *  <LI><code>showInternalFrame(Component parent,java.util.Date date,String title)</code>
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.21 [06-07-2004]
 */
public class CalendarPane extends JPanel implements Form {

	/** 
	 * implementation of MouseAdapter to handle clicking the days
	 * and using the navigation buttons.
	 * <p>Ok and Canceled are handled by FormAction
	 * @see FormAction
	 */
	private class MouseHandler extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			if (e.getSource() instanceof JLabel) {
				selectDay((JLabel) e.getSource());
			} else {
				if (e.getSource() == backYear) {
					current.set(Calendar.YEAR, current.get(Calendar.YEAR) - 1);
				} else if (e.getSource() == backMonth) {
					current.set(
						Calendar.MONTH,
						current.get(Calendar.MONTH) - 1);
				} else if (e.getSource() == forwardYear) {
					current.set(Calendar.YEAR, current.get(Calendar.YEAR) + 1);
				} else if (e.getSource() == forwardMonth) {
					current.set(
						Calendar.MONTH,
						current.get(Calendar.MONTH) + 1);
				}
				makeMonth(current);
				validate();
			}
		}

	}

	private MouseHandler mouseH;
	private JPanel month;
	private JButton backYear;
	private JButton backMonth;
	private JButton forwardYear;
	private JButton forwardMonth;
	private JLabel displayCurrent;
	private JLabel lastSelected;
	private Calendar current;
	

	/**
	 * Constructor for CalendarPane.
	 * @param c Calendar, the date to start the panel at or null for current date
	 */
	public CalendarPane(Calendar c) {
		current = c;
		init();

	}

	/**
	 * initialize the dialog
	 */
	private void init() {
		mouseH = new MouseHandler();
		setLayout(new BorderLayout());
		FormAction fa = new FormAction(
		        this,
		        FormAction.BTN_OK_CANCEL,
		        ResourceBundle.getBundle("org.brains2b.swingext.resource.dialog"));
		if (getRootPane() != null) {
			getRootPane().setDefaultButton(fa.getButton(FormAction.BTN_OK));
		}

		add(fa, BorderLayout.SOUTH);
		if (current == null) {
			current = Calendar.getInstance();
		}
		displayActions();
		makeMonth(current);
	}

	/**
	 * process key bindings overwritten to handle selecting dates through
	 * the keyboard. 
	 * <p>The following keys are supported: 
	 * <LI>Left, Right, Up, Down for selecting the days
	 * <LI>Page-Up, Page-Down for selecting the months
	 * <LI>Alt  Page-Up, Alt Page-Down for selecting the years
	 */
	protected boolean processKeyBinding(
		KeyStroke ks,
		KeyEvent e,
		int condition,
		boolean pressed) {
		if (pressed) {
			if (e.getKeyCode() == KeyEvent.VK_ENTER) {
				performOk();
				e.consume();
				return true;
			} else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
				selectOtherDay(-1);
				e.consume();
				return true;
			} else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
				selectOtherDay(1);
				e.consume();
				return true;
			} else if (e.getKeyCode() == KeyEvent.VK_UP) {
				selectOtherDay(-8);
				e.consume();
				return true;
			} else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
				selectOtherDay(8);
				e.consume();
				return true;
			} else if (e.getKeyCode() == KeyEvent.VK_PAGE_DOWN) {
				if (e.getModifiers() == 8) {
					current.set(Calendar.YEAR, current.get(Calendar.YEAR) - 1);
					backYear.doClick();
					makeMonth(current);
					validate();
				} else {
					current.set(
						Calendar.MONTH,
						current.get(Calendar.MONTH) - 1);
					backMonth.doClick();
					makeMonth(current);
					validate();
				}
				e.consume();
				return true;
			} else if (e.getKeyCode() == KeyEvent.VK_PAGE_UP) {
				if (e.getModifiers() == 8) {
					current.set(Calendar.YEAR, current.get(Calendar.YEAR) + 1);
					forwardYear.doClick();
					makeMonth(current);
					validate();
				} else {
					current.set(
						Calendar.MONTH,
						current.get(Calendar.MONTH) + 1);
					forwardMonth.doClick();
					makeMonth(current);
					validate();
				}
				e.consume();
				return true;
			} else {
				return super.processKeyBinding(ks, e, condition, pressed);

			}
		} else {
			return super.processKeyBinding(ks, e, condition, pressed);

		}

	}

	/* (non-javadoc)
	 * @see org.brains2b.model.gui.Form#performApply()
	 */
	public void performApply() {
		//not implemented
	}

	/* (non-javadoc)
	 * @see org.brains2b.model.gui.Form#performOk()
	 */
	public void performOk() {
        close();
		
	}

	/* (non-javadoc)
	 * @see org.brains2b.model.gui.Form#performUndo()
	 */
	public void performUndo() {
		//not implemented
	}

	/* (non-javadoc)
	 * @see org.brains2b.model.gui.Form#performCancel()
	 */
	public void performCancel() {
		current = null;
		close();

	}

	/**
	 * show a dialog containing this Calendar element and returning a selected date
	 * @param parent Component, a component which provides the hierarchy for the dialog
	 * @param date Date, the date to select initially, if null today date will be used 
	 * @param title String, the title for the dialog window
	 * @return Date, the date selected by the user
	 */
	public static Date showDialog(Component parent, Date date, String title) {
		
		Calendar c = Calendar.getInstance();
		if (date != null) {
			c.setTime(date);
		}
		CalendarPane edit = new CalendarPane(c);
		DialogHelper.showDialog(parent, edit, title);
		if (edit.current != null) {
			return edit.current.getTime();
		} else {
			return null;
		}
	}

	/**
	 * show an internal frame containing this Calendar element and returning a selected date
	 * @param parent Component, a component which provides the hierarchy for the internal frame
	 * @param date Date, the date to select initially, if null today date will be used 
	 * @param title String, the title for the dialog window
	 * @return Date, the date selected by the user
	 */
	public static Date showInternalDialog(
		Component parent,
		Date date,
		String title) {
		
		Calendar c = Calendar.getInstance();
		if (date != null) {
			c.setTime(date);
		}
		CalendarPane edit = new CalendarPane(c);
        DialogHelper.showInternalDialog(parent, edit, title, null);
		
		if (edit.current != null) {
			return edit.current.getTime();
		} else {
			return null;
		}

	}

	/**
	 * Creates a panel with the back and forward buttons for months and years
	 * and the display area for the current month and year.
	 */
	private void displayActions() {
		JPanel p = new JPanel();

		backYear =
			new JButton(
				new ImageIcon(
					getClass().getClassLoader().getResource(
						"org/brains2b/images/Rewind24.gif")));
		backYear.addMouseListener(mouseH);
		backYear.setBorder(BorderFactory.createEtchedBorder());
		backMonth =
			new JButton(
				new ImageIcon(
					getClass().getClassLoader().getResource(
						"org/brains2b/images/Back24.gif")));
		backMonth.addMouseListener(mouseH);
		backMonth.setBorder(BorderFactory.createEtchedBorder());
		displayCurrent = new JLabel("");
		displayCurrent.setPreferredSize(new Dimension(140, 27));
		displayCurrent.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createEtchedBorder(),
				BorderFactory.createEmptyBorder(4, 4, 3, 4)));
		displayCurrent.setHorizontalAlignment(SwingConstants.CENTER);
		forwardYear =
			new JButton(
				new ImageIcon(
					getClass().getClassLoader().getResource(
						"org/brains2b/images/FastForward24.gif")));
		forwardYear.addMouseListener(mouseH);
		forwardYear.setBorder(BorderFactory.createEtchedBorder());
		forwardMonth =
			new JButton(
				new ImageIcon(
					getClass().getClassLoader().getResource(
						"org/brains2b/images/Play24.gif")));
		forwardMonth.addMouseListener(mouseH);
		forwardMonth.setBorder(BorderFactory.createEtchedBorder());

		p.add(backYear);
		p.add(backMonth);
		p.add(displayCurrent);
		p.add(forwardMonth);
		p.add(forwardYear);
		add(p, BorderLayout.NORTH);

	}

	/**
	 * create a JPanel with the complete grid for the month given by the 
	 * current Calendar and set the display of this Month correctly;
	 * @param c Calendar containing the current date
	 */
	private void makeMonth(Calendar c) {
		DateFormatSymbols dfs = new DateFormatSymbols(Locale.getDefault());
		String monthTxt = dfs.getMonths()[c.get(Calendar.MONTH)];
		displayCurrent.setText(monthTxt + " " + c.get(Calendar.YEAR));
		if (month != null)
			remove(month);
		month = new JPanel();
		month.setLayout(new GridLayout(7, 8));
		// blank on 0,0
		month.add(new JLabel(""), 0);
		//days of week
		int weekDays[] = new int[7];
		int first = current.getFirstDayOfWeek();
		for (int i = 0; i < 7; i++) {
			if (i + first > 7)
				first = -i + 1;
			weekDays[i] = i + first;
			month.add(new JLabel(dfs.getShortWeekdays()[i + first]), i + 1);
		}
		int cnt = 1;
		Calendar week = (Calendar) c.clone();
		//week.set(Calendar.DAY_OF_MONTH,1);
		Calendar day = (Calendar) c.clone();
		for (int i = 0; i < 48; i++) {
			//weeknumbers
			if (i % 8 == 0) {
				week.set(Calendar.DAY_OF_MONTH, cnt);
				String weekText = null;
				if (cnt < week.getMaximum(Calendar.DAY_OF_MONTH)) {
					weekText = String.valueOf(week.get(Calendar.WEEK_OF_YEAR));
				}
				JLabel weekLabel = new JLabel(weekText);
				weekLabel.setHorizontalAlignment(SwingConstants.CENTER);
				month.add(weekLabel);
			} else {
				day.set(Calendar.DAY_OF_MONTH, cnt);
				if (weekDays[i % 8 - 1] == day.get(Calendar.DAY_OF_WEEK)
					&& cnt <= c.getActualMaximum(Calendar.DAY_OF_MONTH)) {
					month.add(newDay(cnt++));
				} else {
					month.add(new JLabel(""));
				}
			}
		}
		add(month, BorderLayout.CENTER);
	}

	/**
	 * create a JLabel for a day
	 * @param i int, the value for the Day to create
	 * @return JLabel, the JLabel representing a day, will return it marked selected if
	 * day value is same as day value for current Calendar
	 */
	private JLabel newDay(int i) {
		JLabel day = new JLabel(String.valueOf(i));
		day.setBorder(BorderFactory.createEtchedBorder());
		day.setHorizontalAlignment(SwingConstants.CENTER);
		day.setVerticalAlignment(SwingConstants.CENTER);
		day.addMouseListener(mouseH);
		if (i == current.get(Calendar.DAY_OF_MONTH)) {
			selectDay(day);
		}
		return day;
	}

	/**
	 * handle selection of a Day through the component by changing the borders
	 * on the JLabel and setting the current calendar correctly
	 * @param day JLabel, the JLabel of the day that was selected
	 */
	private void selectDay(JLabel day) {
		int idx = Integer.parseInt(day.getText());
		current.set(Calendar.DAY_OF_MONTH, idx);
		if (lastSelected != null) {
			lastSelected.setBorder(BorderFactory.createEtchedBorder());
		}
		day.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createLineBorder(SystemColor.activeCaption, 2),
				BorderFactory.createEtchedBorder()));
		lastSelected = day;
	}

	/**
	 * select another day given a relative position to the current selected day
	 * @param direction int, the number of days plus or minus the current selected
	 * day
	 */
	private void selectOtherDay(int direction) {

		if (lastSelected != null) {
			int index = -1;
			Component[] cmps = month.getComponents();
			for (int i = 0; i < cmps.length; i++) {
				if (cmps[i] == lastSelected) {
					index = i;
					break;
				}
			}
			index += direction;
			if (index >= 8 && index < cmps.length) {
				if (index % 8 == 0) { //skip the week label
					index += direction;
				}
				JLabel day = (JLabel) month.getComponent(index);
				if (day.getText().length() != 0) {
					if (lastSelected != null) {
						lastSelected.setBorder(
							BorderFactory.createEtchedBorder());
					}
					current.set(
						Calendar.DAY_OF_MONTH,
						Integer.parseInt(day.getText()));
					day.setBorder(
						BorderFactory.createCompoundBorder(
							BorderFactory.createLineBorder(
								SystemColor.activeCaption,
								2),
							BorderFactory.createEtchedBorder()));
					lastSelected = day;
				}
			}
		}
	}
    
    private void close() {
        if (getRootPane().getParent() instanceof JInternalFrame) {
            ((JInternalFrame) getRootPane().getParent()).doDefaultCloseAction();      
        } else {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
    }
}
